from apscheduler.util import undefined
from apscheduler.events import EVENT_ALL
from tornado.options import options


class TaskHelper:
    @staticmethod
    def add_job(func, trigger=None, args=None, kwargs=None, id=None, name=None,
                misfire_grace_time=undefined, coalesce=undefined, max_instances=undefined,
                next_run_time=undefined, jobstore='default', executor='default',
                replace_existing=False, **trigger_args):
        job = options.scheduler.add_job(func, trigger=trigger, args=args, kwargs=kwargs, id=id, name=name,
                                        misfire_grace_time=misfire_grace_time, coalesce=coalesce,
                                        max_instances=max_instances,
                                        next_run_time=next_run_time, jobstore=jobstore, executor=executor,
                                        replace_existing=replace_existing, **trigger_args)
        return job

    @staticmethod
    def scheduled_job(trigger, args=None, kwargs=None, id=None, name=None,
                      misfire_grace_time=undefined, coalesce=undefined, max_instances=undefined,
                      next_run_time=undefined, jobstore='default', executor='default',
                      **trigger_args):
        inner = options.scheduler.scheduled_job(trigger, args=args, kwargs=kwargs, id=id, name=name,
                                                misfire_grace_time=misfire_grace_time, coalesce=coalesce,
                                                max_instances=max_instances,
                                                next_run_time=next_run_time, jobstore=jobstore, executor=executor,
                                                **trigger_args)
        return inner

    @staticmethod
    def modify_job(job_id, jobstore=None, **changes):
        job = options.scheduler.modify_job(job_id, jobstore=jobstore, **changes)
        return job

    @staticmethod
    def reschedule_job(job_id, jobstore=None, trigger=None, **trigger_args):
        job = options.scheduler.reschedule_job(job_id, jobstore=jobstore, trigger=trigger, **trigger_args)
        return job

    @staticmethod
    def pause_job(job_id, jobstore=None):
        job = options.scheduler.pause_job(job_id, jobstore=jobstore)
        return job

    @staticmethod
    def resume_job(job_id, jobstore=None):
        job = options.scheduler.resume_job(job_id, jobstore=jobstore)
        return job

    @staticmethod
    def get_jobs(jobstore=None, pending=None):
        jobs = options.scheduler.get_jobs(jobstore=jobstore, pending=pending)
        return jobs

    @staticmethod
    def get_job(job_id, jobstore=None):
        job = options.scheduler.get_job(job_id, jobstore=jobstore)
        return job

    @staticmethod
    def remove_job(job_id, jobstore=None):
        options.scheduler.remove_job(job_id, jobstore=jobstore)

    @staticmethod
    def remove_all_jobs(jobstore=None):
        options.scheduler.remove_all_jobs(jobstore=jobstore)

    @staticmethod
    def print_jobs(jobstore=None, out=None):
        options.scheduler.print_jobs(jobstore=jobstore, out=out)

    @staticmethod
    def add_executor(executor, alias='default', **executor_opts):
        options.scheduler.add_executor(executor, alias=alias, **executor_opts)

    @staticmethod
    def remove_executor(alias, shutdown=True):
        options.scheduler.remove_executor(alias=alias, shutdown=shutdown)

    @staticmethod
    def add_jobstore(jobstore, alias='default', **jobstore_opts):
        options.scheduler.add_jobstore(jobstore, alias=alias, **jobstore_opts)

    @staticmethod
    def remove_jobstore(alias, shutdown=True):
        options.scheduler.remove_jobstore(alias=alias, shutdown=shutdown)

    @staticmethod
    def add_listener(callback, mask=EVENT_ALL):
        options.scheduler.add_listener(callback, mask=mask)

    @staticmethod
    def remove_listener(callback):
        options.scheduler.remove_listener(callback)



from datetime import datetime
from decimal import Decimal

from sqlalchemy.orm import Session
from tornado.options import options


class SqlAlchemyHelper:
    def __init__(self, pool):
        """
        初始化sql工具
        :param pool: 连接池名称
        """
        if isinstance(pool, str):
            self.pool, self.dialect = options.database[pool]
        else:
            self.pool = pool

    def make_session(self, autocommit=True, **kwargs):
        session = Session(self.pool, autocommit=autocommit, **kwargs)
        return session


def alchemy_default_to_dict(params, data, repetition=False):
    data_list = []
    key_list = []
    if repetition:
        for arg in params:
            if arg.key:
                key_list.append(arg.key)
            else:
                key_list.append(arg.name)
    else:
        for arg in params:
            arg = str(arg)
            if "(" in arg and ")" in arg:
                key_list.append(arg.split(".")[-1][:-1])
            else:
                key_list.append(arg.split(".")[-1])
    if isinstance(data, list):
        for d in data:
            dict_data = dict(zip(key_list, d))
            data_list.append(dict_data)
        return data_list
    else:
        if data:
            return dict(zip(key_list, data))
        else:
            return {}


def sqlalchemy_paging(Query, limit_number, offset_number):
    data_list = Query.limit(limit_number).offset(offset_number).all()
    data_count = Query.count()
    return {"count": data_count, "dataSource": data_list}


def query_set_to_dict(obj):
    obj_dict = {}
    for column in obj.__table__.columns.keys():
        val = getattr(obj, column)
        if isinstance(val, Decimal):
            val = float(val)
        obj_dict[column] = val
    return obj_dict


def query_set_to_list(query_set):
    ret_list = []
    for obj in query_set:
        ret_dict = query_set_to_dict(obj)
        ret_list.append(ret_dict)
    return ret_list


def result_to_json(data):
    if isinstance(data, list):
        return query_set_to_list(data)
    else:
        return query_set_to_dict(data)


def result_page(query, page_num=1, page_size=10):
    offset_number = (page_num - 1) * page_size if page_num >= 1 else 0
    data_list = query.limit(page_size).offset(offset_number).all()
    data_count = query.count()
    return {"count": data_count, "dataSource": data_list}



import inspect

swagger_json_project = []

swagger_json_class = []
python_column_type_dict = {
    "list": "array",
    "str": "string",
    "int": "integer",
    "dict": "object",
    "bool": "boolean",
    "float": "multipleOf"
}
definitions = {}


def parser_swagger(func, desc):
    if type(func).__name__ == "type":
        swagger_json_project.append(
            {
                "desc": desc,
                "route": func.__route_name__,
                "method_list": swagger_json_class.copy()
            }
        )
        swagger_json_class.clear()

    elif type(func).__name__ == "function":
        param_list = [{
            "name": "Authorization",
            "in": "header",
            "required": False,
            "type": "string"
        }]
        args = get_args(func)
        if func.__http_method__ == "get":
            param_list.extend(args)
            swagger_json_class.append({
                "route": func.__route_name__,
                "summary": "Place an order for a pet",

                "operationId": "placeOrder",
                "consumes": ["application/json"],
                "produces": ["application/json", "application/xml"],
                "parameters": param_list,
                "description": desc,
                "http_method": func.__http_method__
            })
        elif func.__http_method__ == "post":
            schema = {
                "type": "object",
                "properties": {
                }
            }
            for param in args:
                param_type = param.get("type")
                if param_type in ["integer", "string"]:
                    schema["properties"][param.get("name", "")] = {
                        "required": param.get("required", ""),
                        "type": param_type,
                        "enum": [param.get("default", "string")]
                    }
                elif param_type in ["array"]:
                    schema["properties"][param.get("name", "")] = {
                        "required": param.get("required", ""),
                        "type": "array",
                        "xml": {"wrapped": True},
                        "items": {"type": "string"},
                        "enum": [param.get("default", "string")]
                    }
                elif param_type in ["object"]:
                    schema["properties"][param.get("name", "")] = {
                        "required": param.get("required", ""),
                        "type": "object",
                        "properties": {

                        }
                    }
            param_list.append({
                "required": True,
                "name": "body",
                "in": "body",
                "schema": schema
            })
            swagger_json_class.append({
                "route": func.__route_name__,
                "parameters": param_list,
                "description": desc,
                "http_method": func.__http_method__
            })
        else:
            pass


def get_args(func):
    params_list = []
    signature = inspect.signature(func)
    for parameter_name, parameter in signature.parameters.items():
        if parameter_name == 'self':
            continue
        else:
            parameter_type = parameter.annotation
            if parameter_type is inspect.Parameter.empty:
                parameter_type = type(parameter.default)
            parameter_default = None
            if parameter.default is not inspect.Parameter.empty:
                parameter_default = parameter.default
            parameter_type_str = "string"
            if not isinstance(parameter_type, type):
                if isinstance(parameter_type, int):
                    parameter_type_str = "integer"
                elif isinstance(parameter_type, str):
                    parameter_type_str = "string"
                elif isinstance(parameter_type, list) or isinstance(parameter_type, tuple):
                    parameter_type_str = "array"
                elif isinstance(parameter_type, dict):
                    parameter_type_str = "object"
                elif issubclass(parameter_type, object):
                    parameter_type_str = "object"
                elif issubclass(parameter_type, bool):
                    parameter_type_str = "boolean"
            params = {
                "required": False if parameter.default is not inspect.Parameter.empty else True,
                "name": parameter_name,
                "in": "query",
                "type": parameter_type_str,
                "default": parameter_default
            }
            params_list.append(params)
    return params_list
